# 日本語ドキュメントは後半にあります。
# akakyouryuu 2014/7/2 I tested this in Ren'Py v6.17 
#
# Automatically composite images define
#
#
# config.automatic_images and config.automatic_images_strip can cause Ren'Py to
# automatically define images. But composite images aren't defined by this.
# 
# I added two config variables config.automatic_images_parts and
# config.automatic_images_not_necessary_parts which cause Ren'Py to
# automatically define composite images.
#
#
# config.automatic_images_parts = None
#
# If this and `config.automatic_images` are not None, this causes Ren'Py to
# automatically also define composite images.
#
# When not set to None, this should be set to a list of strings giving
# directories that include images which are parts of composite images.
#
# Parts are composited from left to right and all parts must be same size.
#
#
# config.automatic_images_not_necessary_parts = []
#
# A list of strings giving directories including images which are not necessary
# parts for composite images.
#
# For example, if relative variables are set like below:
#
# * config.automatic_images = ["/"]
# * config.automatic_images_parts = ["body", "face", "sweat", "surplus"]
# * config.automatic_images_not_necessary_parts = ["sweat"]
#
# and your game directly contains, :
#
# * eileen/body/b1.png
# * eileen/body/b2.png
# * eileen/face/f1.png
# * eileen/face/f2.png
# * eileen/sweat/sweat.png
#
# then below images are defined :
#
# * eileen b1 f1
# * eileen b1 f1 sweat
# * eileen b1 f2
# * eileen b1 f2 sweat
# * eileen b2 f1
# * eileen b2 f1 sweat
# * eileen b2 f2
# * eileen b2 f2 sweat
#
# If you define 'default_<image tag name>', the transform is applied to the
# composite images with the tag. For example, if you define default_eileen
# transform like below,
#
# transform default_eileen:
#     xanchor .5
#
# this transform is applied to composite images with 'eileen' tag.
#
#
# 合成画像自動定義スクリプト
#
# 合成画像(例えば体と表情が別の画像ファイルに保存されているようなもの)を設定に
# 従い、全ての組合せで自動的定義します。
#
#
# .. var:: config.automatic_images_parts = None
#
# これと `config.automatic_images` が None でないなら、 Ren'Pyが自動的に合成画
# 像を定義します。
#
# None を指定するか、合成画像のパーツとなる画像ファイルがあるディレクトリー
# 名の文字列のリストを指定します。画像はリストの左から右の順に合成され、す
# べての画像は同じ大きさでなくてはなりません。
#
# .. var:: config.automatic_images_not_necessary_parts = []
#
# 必ずしも必要というわけではないパーツファイルのあるディレクトリーのリストです。
#     
#
# 例えば関連する変数が以下のように定義され
#
# * config.automatic_images = ["/"]
# * config.automatic_images_parts = ["body", "face", "sweat", "surplus"]
# * config.automatic_images_not_necessary_parts = ["sweat"]
#
# ゲームディレクトリーに以下のファイルがあるなら、
#
# * eileen/body/b1.png
# * eileen/body/b2.png
# * eileen/face/f1.png
# * eileen/face/f2.png
# * eileen/sweat/sweat.png
#
# 次の画像が定義されます。 :
#
# * eileen b1 f1
# * eileen b1 f1 sweat 
# * eileen b1 f2
# * eileen b1 f2 sweat 
# * eileen b2 f1
# * eileen b2 f1 sweat 
# * eileen b2 f2
# * eileen b2 f2 sweat
#
# transform default_<image tag name>:
#     xanchor .5
# のようにすれば合成した画像タグ毎にデフォルトの変換を定義できます。


init -1900 python:

    # directories including images for composite images.
    config.automatic_images_parts = None

    # directories including not necessary images for composite images.
    config.automatic_images_not_necessary_parts = []

init 1900 python hide:

    def create_automatic_composite_images():
        import itertools

        seps = config.automatic_images

        if seps is True:
            seps = [ ' ', '/', '_' ]

        image_parts = dict()

        for dir, fn in renpy.loader.listdirfiles():

            if fn.startswith("_"):
                continue

            # Only .png and .jpg
            if not fn.lower().endswith(".png") and not fn.lower().endswith(".jpg"):
                continue

            # Strip the extension, replace slashes.
            shortfn = fn[:-4].replace("\\", "/")

            #store parts to image_parts
            for i in config.automatic_images_parts:
                if shortfn.find("/"+i+"/") > 0:

                    name = (shortfn[:shortfn.find("/"+i+"/")], )
                    for sep in seps:
                        name = tuple(j for i in name for j in i.split(sep))
                    # Strip name components.
                    while name:
                        for j in config.automatic_images_strip:
                            if name[0] == j:
                                name = name[1:]
                                break
                        else:
                            break

                    parts = (shortfn[shortfn.find("/"+i+"/")+len(i)+2:], )
                    for sep in seps:
                        parts = tuple(j for i in parts for j in i.split(sep))


                    if name not in image_parts:
                        image_parts[name] = { i:[] for i in config.automatic_images_parts }

                    image_parts[name][i].append((parts, fn))

        # define images consisting of several images
        for name in image_parts:
            parts_lists = []
            for part in config.automatic_images_parts:
                if image_parts[name][part]:
                    if part in config.automatic_images_not_necessary_parts:
                        image_parts[name][part].append(None)
                    parts_lists.append(image_parts[name][part])

            all_pattern = map(list, itertools.product(*parts_lists))

            for i in all_pattern:
                name_args = name
                file_args = []
                for k in i:
                    if k is not None:
                        name_args += k[0]
                        file_args.append(k[1])

                # Only names of 2 components or more by default.
                if len(name_args) < config.automatic_images_minimum_components:
                    continue

                # Reject if it already exists.
                if name_args in renpy.display.image.images:
                    continue

                tran = getattr(renpy.store, "default_" + name_args[0], None)
                if tran:
                    renpy.image(name_args, At(Flatten(Fixed(*file_args, fit_first=True)), tran))
                else:
                    renpy.image(name_args, Flatten(Fixed(*file_args, fit_first=True)))
 
    if config.automatic_images and config.automatic_images_parts:
        create_automatic_composite_images()
